@@ -164,7 +164,7 @@ class CovaSimTest(unittest.TestCase):
 
         """
         if not self.simulation_parameters:
-            self.simulation_parameters = parameters.make_pars(set_prognoses=True, prog_by_age=True, use_layers=True)
+            self.simulation_parameters = parameters.make_pars(set_prognoses=True, prog_by_age=True)
         if params_dict:
             self.simulation_parameters.update(params_dict)
         pass
@@ -445,7 +445,6 @@ class TestSupportTests(CovaSimTest):
         """
         All agents start infected
         """
-        self.is_debugging = True
 
         total_agents = 500
         self.set_everyone_infected(agent_count=total_agents)
@@ -461,7 +460,6 @@ class TestSupportTests(CovaSimTest):
         Verifies that there are lots of infections in
         a short time.
         """
-        self.is_debugging = True
         self.assertIsNone(self.simulation_parameters)
         self.assertIsNone(self.sim)
         self.set_smallpop_hightransmission()
